%wrds;
libname rhome slibref=home server=wrds;

rsubmit;
libname home '~';
PROC DATASETS LIB = home; QUIT; RUN;

%include "/home/duke/scottdyr/WT.sas";



/*the dataset dealcomplink was downloaded as an EXCEL file from WRDS on 19Dec2014, converted to a SAS file,
  and uploaded to  my home directory*/


*read in firms that have compustat matches using Robert's data;
proc sql;
   create table dfirms
   as select GVKEY label="Compustat ID",
             bcoid as DSID label="Dealscan ID",
			 facid as FACILITYID label="Dealscan Facility ID",
			 smbl_h as TICKER label="Compustat Ticker",
			 facstartdate as DEALDATE label="Deal Start Date",
			 FIC
   from home.dealcomplink
   order by GVKEY,DEALDATE desc;
   quit;
proc sort data=dfirms;
   by facilityid;
   run;


*check for duplicates;
proc sort data=dfirms noduplicates;
   by gvkey dealdate;
   run;
proc sort data=dfirms nodupkey dupout=checkme;
   by gvkey dsid facilityid;
   run;

*get packageid;
proc sql;
   create table dfirms
   as select a.GVKEY,
             a.DSID,
			 b.PACKAGEID,
			 a.*
   from dfirms as a left join dealscan.facility as b
   on a.facilityid = b.facilityid
   order by gvkey,dealdate;
   quit;

*get the date of the "next" deal;
proc sort data=dfirms nodupkey out=temp;
   by packageid dealdate;
   run;
proc sort data=temp(keep=gvkey packageid dealdate);
   by gvkey descending dealdate;
   run;
data temp;
   set temp;
   by gvkey;
   NDEALDATE = lag(DEALDATE);
   if first.gvkey then NDEALDATE='31DEC2012'd;
   label NDEALDATE = "Next Deal Date";
   format NDEALDATE date9.;   
   run;
proc sql;
   create table dfirms
   as select a.*,
             b.NDEALDATE
   from dfirms as a left join temp as b
   on a.packageid = b.packageid;
   quit;
proc sql;
   drop table temp;
   quit;
*get package information;
proc sql;
   create table dfirms
   as select dfirms.*,
			    package.*
   from dfirms left join dealscan.package
    on dfirms.packageid = package.packageid
	 and dfirms.dealdate = package.dealactivedate;
	quit;

proc sort data=dfirms nodupkey dupout=checkme;
   by gvkey dsid packageid facilityid;
   run;


*get the date of "amendment" if any;
proc sort data=dealscan.dealamendment out=a;
   by packageid amendeddate;
   run;
proc sort data=a nodupkey;
   by packageid;
   run;
proc sql;
   create table dfirms
   as select dfirms.*,
             a.amendeddate as AMENDDATE format=date9.
   from dfirms left join a
   on dfirms.packageid = a.packageid;
   quit;

proc sql;
   drop table a,checkme;
   quit;

*get facility information;
proc sql;
   create table dfirms
   as select a.*,
             b.*
   from dfirms as a left join dealscan.facility as b
   on a.facilityid = b.facilityid;
   quit;

*get pricing information;
data pricing;
   set dealscan.currfacpricing;
   where baserate ne "";
   run;
proc freq data=pricing order=freq;
   tables baserate;
   run;
data pricing;
   set pricing;
   if baserate = "LIBOR" then orderme=1;
     else if baserate="Prime" then orderme=2;
	 else if baserate="Euribor" then orderme=3;
	 else if baserate="CD" then orderme=4;
	 else if baserate="Fixed Rate" then orderme=6;
	 else orderme=5;
	 run;
proc sort data=pricing;
   by facilityid orderme;
   run;
proc sort data=pricing nodupkey;
   by facilityid;
   run;

proc sql;
   create table dfirms
   as select a.*,
             b.BaseRate,
			 b.MinBPS,
			 b.MaxBPS,
			 b.AllInDrawn,
			 b.AllInUndrawn
   from dfirms as a left join pricing as b
   on a.facilityid = b.facilityid;
   quit;

proc sql;
   drop table pricing;
   quit;


*get fees;


*calculate syndicate size as the number of distinct banks participating in the traunch;
proc sql;
   create table temp
   as select facilityid,
             n(distinct LENDER) as SYNDSIZE label="Number of Banks in Syndicate"
   from dealscan.lendershares
   group by facilityid;
   quit;

proc sql;
   create table dfirms
   as select a.*,
             b.*
   from dfirms as a left join temp as b
   on a.facilityid = b.facilityid;
   quit;




/*get the number of prior loans the firm has with the same lead arranger*/
proc sql;
   create table temp
   as select GVKEY,
             DSID,
             packageid,
             facilityid,
			 facilityamt,
			 facilitystartdate,
			 facilityenddate
   from dfirms
   group by packageid
   having facilityamt = max(facilityamt);
   quit;
proc sort data=temp nodupkey dupout=checkme;
   by facilityid;
   quit;


proc sql;
   create table temp
   as select a.*,
             b.*
   from temp as a left join dealscan.lendershares as b
    on a.facilityid = b.facilityid
   group by packageid,a.facilityid;
   quit;

proc freq data=temp;
   where leadarrangercredit="Yes";
   tables lenderrole;
   run;


data temp2;
   set temp;
   where lenderrole = "Admin agent" and LeadArrangerCredit = "Yes";
   run;

proc sort data=temp2 out=temp3(keep=facilityid) nodupkey;
   by facilityid;
   run;
proc sort data=temp;
   by facilityid;
   run;
data temp;
   merge temp(in=a) temp3(in=b);
   by facilityid;
   if a=1 and b=0;
   run;


data temp3;
   set temp;
   where lenderrole in ("Agent" "Arranger" "Bookrunner" "Lead arranger" "Lead bank" "Lead manager")
           and LeadArrangerCredit = "Yes";
   run;

data temp4;
   set temp2 temp3;
   run;
proc sort data=temp4;
   by gvkey companyid facilitystartdate;
   run;
data temp4;
   set temp4;
   by gvkey companyid;
   if first.companyid then PRIORDEALS=-1;
   PRIORDEALS+1;
   label PRIORDEALS = "Number of Prior Deals with This Lead Arranger";
   run;
proc sql;
   create table temp4
   as select packageid,
             max(PRIORDEALS) as PRIORDEALS label="Number of Prior Deals with This Lead Arranger"
   from temp4
   group by packageid;
   quit;
proc sql;
   create table dfirms
   as select dfirms.*,
             temp4.PRIORDEALS
   from dfirms left join temp4
   on dfirms.packageid = temp4.packageid;
   quit;

proc sql;
   drop table temp,temp2,temp3,temp4,checkme;
   quit;


*get the number of financial covenants;
proc sql;
   create table temp
   as select packageid,
             n(distinct covenanttype) as NCOV
   from dealscan.financialcovenant
   group by packageid;
   quit;

*add to it net worth covenants;
proc sql;
  create table temp
  as select COALESCE (a.packageid,b.packageid) as packageid,
            max(0,(NCOV+n(distinct covenanttype))) as NCOV
  from temp as a full join dealscan.networthcovenant as b
   on a.packageid = b.packageid
  group by a.packageid;
  quit;

*merge into main dataset;
proc sql;
   create table dfirms
   as select dfirms.*,
             max(0,NCOV) as NCOV label="Number of Covenants"
   from dfirms left join temp
   on dfirms.packageid = temp.packageid
   order by dsid,packageid,dealdate;
   quit;

proc sql;
   select n(gvkey) as N,
          n(distinct gvkey) as NFIRMS
   from dfirms
   where NCOV > 0;
   quit;




*calculate the Bradley and Roberts Covenant Intensity Index;
data dfirms;
   set dfirms;
   BRINDEX = sum(AssetSalesSweep > 0,EquityIssuanceSweep > 0,DebtIssuanceSweep > 0,DividendRestrictions = "Yes",Secured = "Yes",NCOV > 2);
   label BRINDEX = "Bradley Roberts Covenant Intensity Index";
   array checkme AssetSalesSweep EquityIssuanceSweep DebtIssuanceSweep NCOV;
     do over checkme;
	 if missing(checkme) then BRINDEX = .;
	 end;
	if missing(DividendRestrictions) then BRINDEX = .;
	if missing(Secured) then BRINDEX = .;
   run;





*get covenant targets;
proc sort data=dealscan.financialcovenant out=COV;
   by packageid covenanttype;
   run;

data COV;
   set COV;
   if covenanttype="Max. Capex" then covenanttype="INITIAL_CAPX";
   else if covenanttype="Max. Debt to EBITDA" then covenanttype="INITIAL_DBEBD";
   else if covenanttype="Max. Debt to Equity" then covenanttype="INITIAL_DBNW";
   else if covenanttype="Max. Debt to Tangible Net Worth" then covenanttype="INITIAL_DBTNW";
   else if covenanttype="Max. Leverage ratio" then covenanttype="INITIAL_DBAT";
   else if covenanttype="Max. Loan to Value" then covenanttype="INITIAL_LNV";
   else if covenanttype="Max. Senior Debt to EBITDA" then covenanttype="INITIAL_SDBEBD";
   else if covenanttype="Max. Senior Leverage" then covenanttype="INITIAL_SDBAT";
   else if covenanttype="Min. Cash Interest Coverage" then covenanttype="INITIAL_CICVR";
   else if covenanttype="Min. Current Ratio" then covenanttype="INITIAL_CRTO";
   else if covenanttype="Min. Debt Service Coverage" then covenanttype="INITIAL_DSCVR";
   else if covenanttype="Min. EBITDA" then covenanttype="INITIAL_EBD";
   else if covenanttype="Min. Fixed Charge Coverage" then covenanttype="INITIAL_FCVR";
   else if covenanttype="Min. Interest Coverage" then covenanttype="INITIAL_ICVR";
   else if covenanttype="Min. Quick Ratio" then covenanttype="INITIAL_QRTO";
   else covenanttype=covenanttype;
   if covenanttype = "INITIAL_EBD" then initialratio = initialamt;
   if covenanttype = "INITIAL_CAPX" then initialratio = initialamt;
   run;
proc freq data=cov;
   tables covenanttype;
   run;
proc transpose data=COV out=COV1(drop=_name_ _label_);
   by packageid;
   id covenanttype;
   var initialratio;
   run;
proc sort data=dealscan.financialcovenant out=COV;
   by packageid covenanttype;
   run;
data COV;
   set COV;
   if covenanttype="Max. Capex" then covenanttype="FINAL_CAPX";
   else if covenanttype="Max. Debt to EBITDA" then covenanttype="FINAL_DBEBD";
   else if covenanttype="Max. Debt to Equity" then covenanttype="FINAL_DBNW";
   else if covenanttype="Max. Debt to Tangible Net Worth" then covenanttype="FINAL_DBTNW";
   else if covenanttype="Max. Leverage ratio" then covenanttype="FINAL_DBAT";
   else if covenanttype="Max. Loan to Value" then covenanttype="FINAL_LNV";
   else if covenanttype="Max. Senior Debt to EBITDA" then covenanttype="FINAL_SDBEBD";
   else if covenanttype="Max. Senior Leverage" then covenanttype="FINAL_SDBAT";
   else if covenanttype="Min. Cash Interest Coverage" then covenanttype="FINAL_CICVR";
   else if covenanttype="Min. Current Ratio" then covenanttype="FINAL_CRTO";
   else if covenanttype="Min. Debt Service Coverage" then covenanttype="FINAL_DSCVR";
   else if covenanttype="Min. EBITDA" then covenanttype="FINAL_EBD";
   else if covenanttype="Min. Fixed Charge Coverage" then covenanttype="FINAL_FCVR";
   else if covenanttype="Min. Interest Coverage" then covenanttype="FINAL_ICVR";
   else if covenanttype="Min. Quick Ratio" then covenanttype="FINAL_QRTO";
   else covenanttype=covenanttype;
   if covenanttype = "INITIAL_EBD" then event = final;
   if covenanttype = "INITIAL_CAPX" then event = final;
   run;
proc transpose data=COV out=COV2(drop=_name_ _label_);
   by packageid;
   id covenanttype;
   var event;
   run;
proc sql;
   create table COV
   as select coalesce (COV1.packageid,COV2.packageid) as PACKAGEID,
             INITIAL_CAPX,
			 FINAL_CAPX,
             INITIAL_DBEBD,
			 FINAL_DBEBD,
             INITIAL_DBNW,
			 FINAL_DBNW,
             INITIAL_DBTNW,
			 FINAL_DBTNW,
             INITIAL_DBAT,
			 FINAL_DBAT,
             INITIAL_SDBEBD,
			 FINAL_SDBEBD,
             INITIAL_SDBAT,
			 FINAL_SDBAT,
             INITIAL_CICVR,
			 FINAL_CICVR,
             INITIAL_CRTO,
			 FINAL_CRTO,
             INITIAL_DSCVR,
			 FINAL_DSCVR,
             INITIAL_EBD,
			 FINAL_EBD,
             INITIAL_FCVR,
			 FINAL_FCVR,
             INITIAL_ICVR,
			 FINAL_ICVR,
             INITIAL_QRTO,
			 FINAL_QRTO
    from cov1 full join cov2
	on cov1.packageid = cov2.packageid;
	quit;
*get tangible net worth covenants;
proc sql;
   create table COV
   as select cov.*,
             BASEAMT as INITIAL_TNW label="INITIAL_TNW",
			 PercentOfNetIncome/100 as BUILDUP_TNW label="BUILDUP_TNW"
   from cov left join dealscan.networthcovenant
   on cov.packageid = networthcovenant.packageid
    and networthcovenant.covenanttype = "Tangible Net Worth";
   quit;
*get  net worth covenants;
proc sql;
   create table COV
   as select cov.*,
             BASEAMT as INITIAL_NW label="INITIAL_NW",
			 PercentOfNetIncome/100 as BUILDUP_NW label="BUILDUP_NW"
   from cov left join dealscan.networthcovenant
   on cov.packageid = networthcovenant.packageid
    and networthcovenant.covenanttype = "Net Worth";
   quit;

*merge into main dataset;
proc sql;
   create table dfirms
   as select dfirms.*,
             COV.* 
   from dfirms left join cov
   on dfirms.packageid = cov.packageid;
   quit;

*delete temporary datasets;
proc sql;
   drop table checkme,cov,cov1,cov2,temp;
   quit;

proc sql;
   select n(gvkey) as N,
          n(distinct gvkey) as NFIRMS
   from dfirms
   where NCOV > 0;
   quit;

proc sql;
   create table temp
   as select distinct packageid,
             gvkey,
             NCOV
   from dfirms
   where GVKEY ne "" and packageid ne . and NCOV > 0;
   quit;



*get quarterly compustat data for SLACK computation;
   *definitions are from Table 4 of Demerjian and Owens;
proc sql;
   create table compq
   as select GVKEY,
             CIK,
	     	    intnx('QTR',yyq(FYEARQ,FQTR),0,'END') as FDATEQ format date9. label="Fiscal Quarter",
			    datadate,
			    ACTQ/LCTQ as CRTO label="Current Ratio",
			    (RECTQ+CHEQ)/LCTQ as QRTO label="Quick Ratio",
             (DLTTQ+max(0,DLCQ))/(ATQ-LTQ) as DBNW label="Debt to Equity",
			    (DLTTQ+max(0,DLCQ))/(ATQ-max(0,INTANQ)-LTQ) as DBTNW label="Debt to Tangible Net Worth",
			    (DLTTQ+max(0,DLCQ))/ATQ as DBAT label="Leverage",
			    ATQ-LTQ as NW label="Net Worth",
             (ATQ-max(0,INTANQ)-LTQ) as TNW label="Tangible Net Worth",
             DLTTQ,
             max(0,DLCQ) as DLCQ,
             ATQ,
             NIQ,
			    OIBDPQ,
			    XINTQ,
             INTPNY
   from compm.fundq(where=(indfmt = 'INDL' and datafmt = 'STD' and popsrc = 'D' and consol = 'C' and FYEARQ >= 1985 and missing(datacqtr)=0))
   order by gvkey,fdateq;
	quit;




*bring in the SIC code, which will be used if computing industry standard deviations for denominators;
proc sql;
   create table compq
   as select compq.gvkey,
             compq.CIK,
	          compq.FDATEQ,
	          compq.DATADATE,
			    input(SIC,4.) as SIC,
			    compq.*
   from compq left join compm.company
    on compq.gvkey = company.gvkey;
	quit;

*bring in senior debt and rent expense, which only appear to be available on the annual files;
proc sql;
   create table compq
   as select a.*,
             max(0,b.DS) as DS,
             max(0,b.XRENT) as XRENT
   from compq as a left join compm.funda(where=(indfmt = 'INDL' and datafmt = 'STD' and popsrc = 'D' and consol = 'C')) as b
   on a.gvkey = b.gvkey
    and year(a.fdateq) = b.fyear;
   quit;


proc sort data=compq noduplicates;
   by gvkey fdateq;
   run;
proc sort data=compq nodupkey;
   by gvkey fdateq;
   run;

*de-cumulate flow variables;
proc printto  log=junk; run;
proc expand data=compq method=none out=compq2(where=(datadate ne .)) from=qtr align=end;
   by gvkey;
	id fdateq;
	convert datadate;
	convert INTPNY  = INTPNY_lag1 / transformout = (lag 1) method=none;
	convert DLCQ  = DLCQ_lag1 / transformout = (lag 1) method=none;
	run;
proc printto;run;
data compq2;
   set compq2;
   if qtr(fdateq) > 1 then INTPNY = INTPNY - INTPNY_lag1;
      else INTPNY = INTPNY;
   run;

*annualize key flow variables;
proc printto  log=junk; run;
proc expand data=compq2 method=none out=compq2(where=(datadate ne .)) from=qtr align=end;
   by gvkey;
	id fdateq;
	convert datadate;
	convert OIBDPQ  = OIBDPQ / transformout = (nomiss movsum 4 trimleft 3) method=none;
	convert XINTQ  = XINTQ / transformout = (nomiss movsum 4 trimleft 3) method=none;
	convert INTPNY  = INTPNQ / transformout = (nomiss movsum 4 trimleft 3) method=none;
	run;
proc printto;run;


*calculate the fifteen covenant actuals;
proc sql;
   create table compq2
   as select GVKEY,
             CIK,
             FDATEQ,
             DATADATE,
             SIC,
             ATQ,
             NIQ,
             OIBDPQ/XINTQ as ICVR label="Interest Coverage",
             OIBDPQ/INTPNQ as CICVR label="Cash Interest Coverage",
             OIBDPQ/(XINTQ+DLCQ_lag1+XRENT) as FCVR label="Fixed Charge Coverage",
             OIBDPQ/(XINTQ+DLCQ_lag1) as DSCVR label="Debt Service Coverage",
             (DLTTQ+DLCQ)/OIBDPQ as DBEBD label="Debt to EBITDA",
             (DLTTQ+DLCQ-DS)/OIBDPQ as SDBEBD label="Senior Debt to EBITDA",
             DBAT,
             (DLTTQ+DLCQ-DS)/ATQ as SDBAT label="Senior Leverage",
             DBTNW,
             DBNW,
             CRTO,
             QRTO,
             OIBDPQ as EBD label="EBITDA",
             NW,
             TNW
   from compq2
   order by gvkey,fdateq;
   quit;


proc sort data=compq2;
   by gvkey fdateq;
   run;

*Get some dummy variables that will be used to make sure the
   moving standard deviation has the most recent three years
   as valid observations;
data compq2;
   set compq2;
   array indicate(*) ICVR CICVR FCVR DSCVR DBEBD SDBEBD DBAT SDBAT DBTNW DBNW CRTO QRTO EBD NW TNW;
   array indicate2(*) I_ICVR I_CICVR I_FCVR I_DSCVR I_DBEBD I_SDBEBD I_DBAT I_SDBAT I_DBTNW I_DBNW I_CRTO I_QRTO I_EBD I_NW I_TNW;
   do i=1 to dim(indicate);
    indicate2(i) = missing(indicate(i))=0;
	end;
   drop i;
  run;
*calculate the SDBTNW standard deviations;
proc printto  log=junk; run;
proc expand data=compq2 method=none out=compq2(where=(datadate ne .)) from=qtr align=end;
   by gvkey;
	id fdateq;
	convert datadate;

   convert ICVR  = STD_ICVR / transformout = (movstd 8 trimleft 2) method=none;
	convert I_ICVR  = I_ICVR / transformout = (nomiss movsum 3 trimleft 2) method=none;

   convert CICVR  = STD_CICVR / transformout = (movstd 8 trimleft 2) method=none;
	convert I_CICVR  = I_CICVR / transformout = (nomiss movsum 3 trimleft 2) method=none;

   convert FCVR  = STD_FCVR / transformout = (movstd 8 trimleft 2) method=none;
	convert I_FCVR  = I_FCVR / transformout = (nomiss movsum 3 trimleft 2) method=none;

   convert DSCVR  = STD_DSCVR / transformout = (movstd 8 trimleft 2) method=none;
	convert I_DSCVR  = I_DSCVR / transformout = (nomiss movsum 3 trimleft 2) method=none;

	convert DBEBD  = STD_DBEBD / transformout = (movstd 8 trimleft 2) method=none;
	convert I_DBEBD  = I_DBEBD / transformout = (nomiss movsum 3 trimleft 2) method=none;

	convert SDBEBD  = STD_SDBEBD / transformout = (movstd 8 trimleft 2) method=none;
	convert I_SDBEBD  = I_SDBEBD / transformout = (nomiss movsum 3 trimleft 2) method=none;

	convert DBAT  = STD_DBAT / transformout = (movstd 8 trimleft 2) method=none;
	convert I_DBAT  = I_DBAT / transformout = (nomiss movsum 3 trimleft 2) method=none;

	convert SDBAT  = STD_SDBAT / transformout = (movstd 8 trimleft 2) method=none;
	convert I_SDBAT  = I_SDBAT / transformout = (nomiss movsum 3 trimleft 2) method=none;

	convert DBTNW  = STD_DBTNW / transformout = (movstd 8 trimleft 2) method=none;
	convert I_DBTNW  = I_DBTNW / transformout = (nomiss movsum 3 trimleft 2) method=none;

	convert DBNW  = STD_DBNW / transformout = (movstd 8 trimleft 2) method=none;
	convert I_DBNW  = I_DBNW / transformout = (nomiss movsum 3 trimleft 2) method=none;


   convert CRTO  = STD_CRTO / transformout = (movstd 8 trimleft 2) method=none;
	convert I_CRTO  = I_CRTO / transformout = (nomiss movsum 3 trimleft 2) method=none;

	convert QRTO  = STD_QRTO / transformout = (movstd 8 trimleft 2) method=none;
	convert I_QRTO  = I_QRTO / transformout = (nomiss movsum 3 trimleft 2) method=none;

	convert EBD  = STD_EBD / transformout = (movstd 8 trimleft 2) method=none;
	convert I_EBD  = I_EBD / transformout = (nomiss movsum 3 trimleft 2) method=none;

	convert NW  = STD_NW / transformout = (movstd 8 trimleft 2) method=none;
	convert I_NW  = I_NW / transformout = (nomiss movsum 3 trimleft 2) method=none;

	convert TNW  = STD_TNW / transformout = (movstd 8 trimleft 2) method=none;
	convert I_TNW  = I_TNW / transformout = (nomiss movsum 3 trimleft 2) method=none;
run;
proc printto;run;


*only use standard deviations that are based on at least the three most current year;
data compq2;
   set compq2;
   array indicate(*) STD_ICVR STD_CICVR STD_FCVR STD_DSCVR STD_DBEBD STD_SDBEBD STD_DBAT STD_SDBAT STD_DBTNW STD_DBNW STD_CRTO STD_QRTO STD_EBD STD_NW STD_TNW;
   array indicate2(*) I_ICVR I_CICVR I_FCVR I_DSCVR I_DBEBD I_SDBEBD I_DBAT I_SDBAT I_DBTNW I_DBNW I_CRTO I_QRTO I_EBD I_NW I_TNW;
   do i=1 to dim(indicate);
    if indicate2(i) < 3 then indicate(i)=.;
	end;
   drop i I_:;
  run;

*rank into 30 size groups;
proc rank data=compq2 groups=30 out=compq2;
   var ATQ;
   ranks ATQ_rank30;
   run;

data compq3;
   set compq2;
   run;

*delete observations where the financial ratio is negative;
%macro deleteme(vars=);
%let k=1;
%let v=%scan(&vars, &k);
 %do %while(&v NE);
 %put working on variable: &v;
   data compq3;
      set compq3;
      if missing(&v) then delete;
      run;
 %let k=%eval(&k+1);
 %let v=%scan(&vars, &k);
 %end;
%mend;
%deleteme(vars=ICVR CICVR FCVR DSCVR DBEBD SDBEBD DBAT SDBAT DBTNW DBNW CRTO QRTO EBD NW TNW);
%macro deleteme2(vars=);
%let k=1;
%let v=%scan(&vars, &k);
 %do %while(&v NE);
 %put working on variable: &v;
   data compq3;
      set compq3;
      if &v < 0 then delete;
      run;
 %let k=%eval(&k+1);
 %let v=%scan(&vars, &k);
 %end;
%mend;
%deleteme2(vars=ICVR CICVR FCVR DSCVR DBEBD SDBEBD DBAT SDBAT DBTNW DBNW CRTO QRTO EBD NW TNW);





*get lags so i can compute chanages;
proc printto  log=junk; run;
proc expand data=compq3 method=none out=compq3(where=(datadate ne .)) from=qtr align=end;
   by gvkey;
	id fdateq;
	convert datadate;
	convert ICVR  = ICVR_lag1 / transformout = (lag 1) method=none;
	convert CICVR  = CICVR_lag1 / transformout = (lag 1) method=none;
	convert FCVR  = FCVR_lag1 / transformout = (lag 1) method=none;
	convert DSCVR  = DSCVR_lag1 / transformout = (lag 1) method=none;
	convert DBEBD  = DBEBD_lag1 / transformout = (lag 1) method=none;
	convert SDBEBD  = SDBEBD_lag1 / transformout = (lag 1) method=none;
	convert DBAT  = DBAT_lag1 / transformout = (lag 1) method=none;
	convert SDBAT  = SDBAT_lag1 / transformout = (lag 1) method=none;
	convert DBTNW  = DBTNW_lag1 / transformout = (lag 1) method=none;
	convert DBNW  = DBNW_lag1 / transformout = (lag 1) method=none;
	convert CRTO  = CRTO_lag1 / transformout = (lag 1) method=none;
	convert QRTO  = QRTO_lag1 / transformout = (lag 1) method=none;
	convert EBD  = EBD_lag1 / transformout = (lag 1) method=none;
	convert NW  = NW_lag1 / transformout = (lag 1) method=none;
	convert TNW  = TNW_lag1 / transformout = (lag 1) method=none;
	run;
proc printto;run;


*combine data;
proc sql;
   create table compq3
   as select GVKEY,
             CIK,
             FDATEQ,
             DATADATE,
             SIC,
             ATQ,
             NIQ,
             ATQ_rank30,
             ICVR,
             ICVR/ICVR_lag1 as PCHG1_ICVR,
             STD_ICVR,

             CICVR,
             CICVR/CICVR_lag1 as PCHG1_CICVR,
             STD_CICVR,

             FCVR,
             FCVR/FCVR_lag1 as PCHG1_FCVR,
             STD_FCVR,

             DSCVR,
             DSCVR/DSCVR_lag1 as PCHG1_DSCVR,
             STD_DSCVR,

             DBEBD,
             DBEBD/DBEBD_lag1 as PCHG1_DBEBD,
             STD_DBEBD,

             SDBEBD,
             SDBEBD/SDBEBD_lag1 as PCHG1_SDBEBD,
             STD_SDBEBD,

             DBAT,
             DBAT/DBAT_lag1 as PCHG1_DBAT,
             STD_DBAT,

             SDBAT,
             SDBAT/SDBAT_lag1 as PCHG1_SDBAT,
             STD_SDBAT,

             DBTNW,
             DBTNW/DBTNW_lag1 as PCHG1_DBTNW,
             STD_DBTNW,

             DBNW,
             DBNW/DBNW_lag1 as PCHG1_DBNW,
             STD_DBNW,

             CRTO,
             CRTO/CRTO_lag1 as PCHG1_CRTO,
             STD_CRTO,

             QRTO,
             QRTO/QRTO_lag1 as PCHG1_QRTO,
             STD_QRTO,

             EBD,
             EBD/EBD_lag1 as PCHG1_EBD,
             STD_EBD,

             NW,
             NW/NW_lag1 as PCHG1_NW,
             STD_NW,

             TNW,
             TNW/TNW_lag1 as PCHG1_TNW,
             STD_TNW
    from compq3;
    quit;


*delete obs without all 15 change ratios;
data compq4;
   set compq3;
   array needme2 PCHG1_ICVR PCHG1_CICVR PCHG1_FCVR PCHG1_DSCVR PCHG1_DBEBD PCHG1_SDBEBD PCHG1_DBAT PCHG1_SDBAT
                PCHG1_DBTNW PCHG1_DBNW PCHG1_CRTO PCHG1_QRTO PCHG1_EBD PCHG1_NW PCHG1_TNW;
   do over needme2;
     if missing(needme2) then delete;
     end;
   run;

*truncate all change ratios at 1 and 99;
%wt(data=compq4,pctl=1 99, type=T, drop=Y, vars=PCHG1_ICVR PCHG1_CICVR PCHG1_FCVR PCHG1_DSCVR PCHG1_DBEBD PCHG1_SDBEBD PCHG1_DBAT PCHG1_SDBAT
                                                PCHG1_DBTNW PCHG1_DBNW PCHG1_CRTO PCHG1_QRTO PCHG1_EBD PCHG1_NW PCHG1_TNW);



*create the covariance matrix for each of the size groups;
proc sort data=compq4;
   by ATQ_rank30;
   run;
proc corr data=compq4 cov nocorr nosimple outp=COVMAT noprint;
   by ATQ_rank30;
   var PCHG1_ICVR PCHG1_CICVR PCHG1_FCVR PCHG1_DSCVR PCHG1_DBEBD PCHG1_SDBEBD PCHG1_DBAT PCHG1_SDBAT
                PCHG1_DBTNW PCHG1_DBNW PCHG1_CRTO PCHG1_QRTO PCHG1_EBD PCHG1_NW PCHG1_TNW;
   run;

*create standard deviations of each of the measures by size group;
proc means noprint data=compq4;
   by ATQ_rank30;
   var ICVR CICVR FCVR DSCVR DBEBD SDBEBD DBAT SDBAT DBTNW DBNW CRTO QRTO EBD NW TNW;
   output out=temp(drop=_type_ _freq_) std= ISTD_ICVR ISTD_CICVR ISTD_FCVR ISTD_DSCVR ISTD_DBEBD ISTD_SDBEBD ISTD_DBAT ISTD_SDBAT ISTD_DBTNW ISTD_DBNW ISTD_CRTO ISTD_QRTO ISTD_EBD ISTD_NW ISTD_TNW;
   run;
proc sql;
   create table compq2
   as select a.*,
             b.*
   from compq2 as a left join temp as b
   on a.atq_rank30 = b.atq_rank30;
   quit;

proc sql;
   drop table temp;
   quit;




*merge the compustat data with the dealscan data;

*go make the covenant data a time series and get event time;
   *use the facility with the longest maturity. if a package has two facilities that end on samedate, use the largest facility
    then if still have duplicates, use the one with the lowest facilityid;
proc sql;
   create table dfirms2
   as select a.*
   from dfirms as a
   where packageid ne .
   group by packageid
   having facilityenddate = max(facilityenddate);
   quit;
proc sql;
   create table dfirms2
   as select a.*
   from dfirms2 as a
   group by packageid
   having facilityamt = max(facilityamt);
   quit;
proc sort data=dfirms2;
   by packageid facilityid;
   run;
proc sort data=dfirms2 nodupkey dupout=checkme;
   by packageid;
   run;

proc sql;
   create table temp
   as select a.*,
             b.*
   from dfirms2(keep=gvkey packageid dealdate facilitystartdate facilityenddate) as a, compq2(keep=gvkey fdateq datadate) as b
   where a.gvkey = b.gvkey
   order by gvkey,packageid,fdateq;
   quit;
*get quarter number;
proc sql;
   create table temp
   as select a.*,
             intck('QTR',min(fdateq),fdateq) as QNUM
   from temp(keep=gvkey packageid fdateq datadate dealdate facilitystartdate facilityenddate) as a
   where packageid ne .
   group by packageid
   order by packageid,fdateq;
   quit;
proc sql;
   create table temp2
   as select a.*
   from temp as a
   where dealdate <= datadate <= dealdate+91
   group by packageid
   having datadate = min(datadate)
   order by packageid;
   quit;
*get ETIME;
proc sql;
   create table temp3
   as select a.*,
             a.QNUM-b.QNUM+1as ETIME
   from temp as a left join temp2 as b
    on a.packageid = b.packageid
   order by packageid,fdateq;
   quit;
data temp3;
   set temp3;
   where -8 <= etime and datadate <= facilityenddate;
   run;

proc sql;
   create table dfirms2
   as select a.*,
             b.*
   from temp3 as a left join dfirms2 as b
   on a.packageid = b.packageid
   order by packageid,fdateq;
   quit;


*get the estimated covenant after adjusting linearly for covenant changes;
proc sql;
   create table check
   as select distinct packageid,
             intck('QTR',facilitystartdate,facilityenddate) as Z
   from dfirms2;
   quit;

proc sort data=check out=check noduplicates;
   by packageid;
   run;
proc sort data=check out=check nodupkey;
   by packageid;
   run;
proc sql;
   create table dfirms3
   as select dfirms2.GVKEY,
             dfirms2.DSID,
			    dfirms2.PACKAGEID,
			    dfirms2.TICKER,
			    dfirms2.DATADATE,
			    dfirms2.FDATEQ,
			    dfirms2.ETIME,

			 INITIAL_ICVR,
			 case when dfirms2.ETIME <= 1 then INITIAL_ICVR
			      when dfirms2.ETIME >= check.Z and FINAL_ICVR ne . then FINAL_ICVR
				  when 1 < dfirms2.ETIME < check.Z and FINAL_ICVR ne . then (FINAL_ICVR-INITIAL_ICVR)*dfirms2.ETIME/check.Z+INITIAL_ICVR
				  else INITIAL_ICVR
			end as TARGET_ICVR,

			 INITIAL_CICVR,
			 case when dfirms2.ETIME <= 1 then INITIAL_CICVR
			      when dfirms2.ETIME >= check.Z and FINAL_CICVR ne . then FINAL_CICVR
				  when 1 < dfirms2.ETIME < check.Z and FINAL_CICVR ne . then (FINAL_CICVR-INITIAL_CICVR)*dfirms2.ETIME/check.Z+INITIAL_CICVR
				  else INITIAL_CICVR
			end as TARGET_CICVR,

			 INITIAL_FCVR,
			 case when dfirms2.ETIME <= 1 then INITIAL_FCVR
			      when dfirms2.ETIME >= check.Z and FINAL_FCVR ne . then FINAL_FCVR
				  when 1 < dfirms2.ETIME < check.Z and FINAL_FCVR ne . then (FINAL_FCVR-INITIAL_FCVR)*dfirms2.ETIME/check.Z+INITIAL_FCVR
				  else INITIAL_FCVR
			end as TARGET_FCVR,

			 INITIAL_DSCVR,
			 case when dfirms2.ETIME <= 1 then INITIAL_DSCVR
			      when dfirms2.ETIME >= check.Z and FINAL_DSCVR ne . then FINAL_DSCVR
				  when 1 < dfirms2.ETIME < check.Z and FINAL_DSCVR ne . then (FINAL_DSCVR-INITIAL_DSCVR)*dfirms2.ETIME/check.Z+INITIAL_DSCVR
				  else INITIAL_DSCVR
			end as TARGET_DSCVR,

			 INITIAL_DBEBD,
			 case when dfirms2.ETIME <= 1 then INITIAL_DBEBD
			      when dfirms2.ETIME >= check.Z and FINAL_DBEBD ne . then FINAL_DBEBD
				  when 1 < dfirms2.ETIME < check.Z and FINAL_DBEBD ne . then (FINAL_DBEBD-INITIAL_DBEBD)*dfirms2.ETIME/check.Z+INITIAL_DBEBD
				  else INITIAL_DBEBD
			end as TARGET_DBEBD,

			 INITIAL_SDBEBD,
			 case when dfirms2.ETIME <= 1 then INITIAL_SDBEBD
			      when dfirms2.ETIME >= check.Z and FINAL_SDBEBD ne . then FINAL_SDBEBD
				  when 1 < dfirms2.ETIME < check.Z and FINAL_SDBEBD ne . then (FINAL_SDBEBD-INITIAL_SDBEBD)*dfirms2.ETIME/check.Z+INITIAL_SDBEBD
				  else INITIAL_SDBEBD
			end as TARGET_SDBEBD,

			 INITIAL_DBAT,
			 case when dfirms2.ETIME <= 1 then INITIAL_DBAT
			      when dfirms2.ETIME >= check.Z and FINAL_DBAT ne . then FINAL_DBAT
				  when 1 < dfirms2.ETIME < check.Z and FINAL_DBAT ne . then (FINAL_DBAT-INITIAL_DBAT)*dfirms2.ETIME/check.Z+INITIAL_DBAT
				  else INITIAL_DBAT
			end as TARGET_DBAT,

			 INITIAL_SDBAT,
			 case when dfirms2.ETIME <= 1 then INITIAL_SDBAT
			      when dfirms2.ETIME >= check.Z and FINAL_SDBAT ne . then FINAL_SDBAT
				  when 1 < dfirms2.ETIME < check.Z and FINAL_SDBAT ne . then (FINAL_SDBAT-INITIAL_SDBAT)*dfirms2.ETIME/check.Z+INITIAL_SDBAT
				  else INITIAL_SDBAT
			end as TARGET_SDBAT,

			 INITIAL_DBTNW,
			 case when dfirms2.ETIME <= 1 then INITIAL_DBTNW
			      when dfirms2.ETIME >= check.Z and FINAL_DBTNW ne . then FINAL_DBTNW
				  when 1 < dfirms2.ETIME < check.Z and FINAL_DBTNW ne . then (FINAL_DBTNW-INITIAL_DBTNW)*dfirms2.ETIME/check.Z+INITIAL_DBTNW
				  else INITIAL_DBTNW
			end as TARGET_DBTNW,

			 INITIAL_DBNW,
			 case when dfirms2.ETIME <= 1 then INITIAL_DBNW
			      when dfirms2.ETIME >= check.Z and FINAL_DBNW ne . then FINAL_DBNW
				  when 1 < dfirms2.ETIME < check.Z and FINAL_DBNW ne . then (FINAL_DBNW-INITIAL_DBNW)*dfirms2.ETIME/check.Z+INITIAL_DBNW
				  else INITIAL_DBNW
			end as TARGET_DBNW,

           INITIAL_CRTO,
			 case when dfirms2.ETIME <= 1 then INITIAL_CRTO
			      when dfirms2.ETIME >= check.Z and FINAL_CRTO ne . then FINAL_CRTO
				  when 1 < dfirms2.ETIME < check.Z and FINAL_CRTO ne . then (FINAL_CRTO-INITIAL_CRTO)*dfirms2.ETIME/check.Z+INITIAL_CRTO
				  else INITIAL_CRTO
			end as TARGET_CRTO,

			 INITIAL_QRTO,
			 case when dfirms2.ETIME <= 1 then INITIAL_QRTO
			      when dfirms2.ETIME >= check.Z and FINAL_QRTO ne . then FINAL_QRTO
				  when 1 < dfirms2.ETIME < check.Z and FINAL_QRTO ne . then (FINAL_QRTO-INITIAL_QRTO)*dfirms2.ETIME/check.Z+INITIAL_QRTO
				  else INITIAL_QRTO
			end as TARGET_QRTO,

			 INITIAL_EBD,
			 case when dfirms2.ETIME <= 1 then INITIAL_EBD
			      when dfirms2.ETIME >= check.Z and FINAL_EBD ne . then FINAL_EBD
				  when 1 < dfirms2.ETIME < check.Z and FINAL_EBD ne . then (FINAL_EBD-INITIAL_EBD)*dfirms2.ETIME/check.Z+INITIAL_EBD
				  else INITIAL_EBD
			end as TARGET_EBD,

			  INITIAL_CAPX,
			 case when dfirms2.ETIME <= 1 then INITIAL_CAPX
			      when dfirms2.ETIME >= check.Z and FINAL_CAPX ne . then FINAL_CAPX
				  when 1 < dfirms2.ETIME < check.Z and FINAL_CAPX ne . then (FINAL_CAPX-INITIAL_CAPX)*dfirms2.ETIME/check.Z+INITIAL_CAPX
				  else INITIAL_CAPX
			end as TARGET_CAPX

   from dfirms2,check
   where dfirms2.packageid = check.packageid;
   quit;


proc sql;
   create table dfirms4
   as select dfirms2.GVKEY,
             dfirms2.DSID,
			 dfirms2.PACKAGEID,
			 dfirms2.TICKER,
			 dfirms2.DATADATE,
			 dfirms2.FDATEQ,
			 dfirms2.ETIME,
			 dfirms2.INITIAL_TNW,
			 dfirms2.BUILDUP_TNW,
			 dfirms2.INITIAL_NW,
			 dfirms2.BUILDUP_NW
   from dfirms2 
   order by gvkey,PACKAGEID,fdateq;
   quit;
proc sql;
   create table dfirms4
   as select a.*,
             b.NIQ
   from dfirms4 as a left join compq3 as b
   on a.gvkey = b.gvkey
   and a.fdateq = b.fdateq
   order by gvkey,PACKAGEID,fdateq;
   quit;


data dfirms5;
   set dfirms4;
   by gvkey PACKAGEID fdateq;
   if first.PACKAGEID then TARGET_TNW = INITIAL_TNW;
     if ETIME <=1 then TARGET_TNW = INITIAL_TNW;
	 if ETIME > 1 and BUILDUP_TNW ne . and NIQ > 0 then TARGET_TNW = TARGET_TNW+(NIQ*1000000*BUILDUP_TNW);
	 if ETIME > 1 and (BUILDUP_TNW = . or NIQ <= 0) then TARGET_TNW = TARGET_TNW;

   if first.PACKAGEID then TARGET_NW = INITIAL_NW;
     if ETIME <=1 then TARGET_NW = INITIAL_NW;
	 if ETIME > 1 and BUILDUP_NW ne . and NIQ > 0 then TARGET_NW = TARGET_NW+(NIQ*1000000*BUILDUP_NW);
	 if ETIME > 1 and (BUILDUP_NW = . or NIQ <= 0) then TARGET_NW = TARGET_NW;

  retain TARGET_TNW TARGET_NW;
  run;


proc sql;
   create table dfirms6
   as select dfirms2.*,
             dfirms3.*
   from dfirms2(keep=GVKEY--NCOV) left join dfirms3
   on dfirms2.PACKAGEID = dfirms3.PACKAGEID
     and dfirms2.fdateq = dfirms3.fdateq;
   quit;


proc sql;
   create table dfirms6
   as select dfirms6.*,
			 dfirms5.INITIAL_NW,
             dfirms5.TARGET_NW,
             dfirms5.INITIAL_TNW,
             dfirms5.TARGET_TNW
   from dfirms6, dfirms5
   where dfirms6.PACKAGEID = dfirms5.PACKAGEID
    and dfirms6.fdateq = dfirms5.fdateq;
   quit;


proc sql;
   drop table dfirms2,dfirms3,dfirms4,dfirms5,check,checkme,temp,temp2,temp3;
   quit;


*merge in compustat actuals and calculate SLACK for each varaible;
proc sql;
   create table dfirms7
   as select a.gvkey,
             c.cik,
             c.ATQ_rank30,
             a.*,

			 b.INITIAL_CAPX,
			 b.TARGET_CAPX,

			 b.INITIAL_ICVR,
			 b.TARGET_ICVR,
			 c.ICVR as ACTUAL_ICVR,
			 c.STD_ICVR,
			 (c.ICVR-b.TARGET_ICVR)/STD_ICVR as SLACK1_ICVR label="Standardized Interest Coverage Ratio Slack",
			 (c.ICVR-b.TARGET_ICVR) as SLACK2_ICVR label="Interest Coverage Ratio Slack",
			 log(c.ICVR)-log(b.TARGET_ICVR) as SLACK3_ICVR label="Log Interest Coverage Ratio Slack",
			 (c.ICVR-b.TARGET_ICVR)/ISTD_ICVR as SLACK4_ICVR label="IStandardized Interest Coverage Ratio Slack",
			 (c.ICVR-b.TARGET_ICVR)/b.TARGET_ICVR as SLACK5_ICVR label="Percent Interest Coverage Ratio Slack",

			 b.INITIAL_CICVR,
			 b.TARGET_CICVR,
			 c.CICVR as ACTUAL_CICVR,
			 c.STD_CICVR,
			 (c.CICVR-b.TARGET_CICVR)/STD_CICVR as SLACK1_CICVR label="Standardized Cash Interest Coverage Ratio Slack",
			 (c.CICVR-b.TARGET_CICVR) as SLACK2_CICVR label="Cash Interest Coverage Ratio Slack",
			 log(c.CICVR)-log(b.TARGET_CICVR) as SLACK3_CICVR label="Log Cash Interest Coverage Ratio Slack",
			 (c.CICVR-b.TARGET_CICVR)/ISTD_CICVR as SLACK4_CICVR label="IStandardized Cash Interest Coverage Ratio Slack",
			 (c.CICVR-b.TARGET_CICVR)/b.TARGET_CICVR as SLACK5_CICVR label="Percent Cash Interest Coverage Ratio Slack",

			 b.INITIAL_FCVR,
			 b.TARGET_FCVR,
			 c.FCVR as ACTUAL_FCVR,
			 c.STD_FCVR,
			 (c.FCVR-b.TARGET_FCVR)/STD_FCVR as SLACK1_FCVR label="Standardized Fixed Charge Coverage Ratio Slack",
			 (c.FCVR-b.TARGET_FCVR) as SLACK2_FCVR label="Fixed Charge Coverage Ratio Slack",
			 log(c.FCVR)-log(b.TARGET_FCVR) as SLACK3_FCVR label="Log Fixed Charge Coverage Ratio Slack",
			 (c.FCVR-b.TARGET_FCVR)/ISTD_FCVR as SLACK4_FCVR label="IStandardized Fixed Charge Coverage Ratio Slack",
			 (c.FCVR-b.TARGET_FCVR)/b.TARGET_FCVR as SLACK5_FCVR label="Percent Fixed Charge Coverage Ratio Slack",

			 b.INITIAL_DSCVR,
			 b.TARGET_DSCVR,
			 c.DSCVR as ACTUAL_DSCVR,
			 c.STD_DSCVR,
			 (c.DSCVR-b.TARGET_DSCVR)/STD_DSCVR as SLACK1_DSCVR label="Standardized Debt Service Coverage Ratio Slack",
			 (c.DSCVR-b.TARGET_DSCVR) as SLACK2_DSCVR label="Debt Service Coverage Ratio Slack",
			 log(c.DSCVR)-log(b.TARGET_DSCVR) as SLACK3_DSCVR label="Log Debt Service Coverage Ratio Slack",
			 (c.DSCVR-b.TARGET_DSCVR)/ISTD_DSCVR as SLACK4_DSCVR label="IStandardized Debt Service Coverage Ratio Slack",
			 (c.DSCVR-b.TARGET_DSCVR)/b.TARGET_DSCVR as SLACK5_DSCVR label="Percent Debt Service Coverage Ratio Slack",

			 b.INITIAL_DBEBD,
			 b.TARGET_DBEBD,
			 c.DBEBD as ACTUAL_DBEBD,
			 c.STD_DBEBD,
			 (c.DBEBD-b.TARGET_DBEBD)*-1/STD_DBEBD as SLACK1_DBEBD label="Standardized Debt to EBITDA Slack",
			 (c.DBEBD-b.TARGET_DBEBD)*-1 as SLACK2_DBEBD label="Debt to EBITDA Slack",
			 -log(c.DBEBD)+log(b.TARGET_DBEBD) as SLACK3_DBEBD label="Log Debt to EBITDA Slack",
			 (c.DBEBD-b.TARGET_DBEBD)*-1/ISTD_DBEBD as SLACK4_DBEBD label="IStandardized Debt to EBITDA Slack",
			 (c.DBEBD-b.TARGET_DBEBD)*-1/b.TARGET_DBEBD as SLACK5_DBEBD label="Percent Debt to EBITDA Slack",

          b.INITIAL_SDBEBD,
			 b.TARGET_SDBEBD,
			 c.SDBEBD as ACTUAL_SDBEBD,
			 c.STD_SDBEBD,
			 (c.SDBEBD-b.TARGET_SDBEBD)*-1/STD_SDBEBD as SLACK1_SDBEBD label="Standardized Senior Debt to EBITDA Slack",
			 (c.SDBEBD-b.TARGET_SDBEBD)*-1 as SLACK2_SDBEBD label="Senior Debt to EBITDA Slack",
			 -log(c.SDBEBD)+log(b.TARGET_SDBEBD) as SLACK3_SDBEBD label="Log Senior Debt to EBITDA Slack",
			 (c.SDBEBD-b.TARGET_SDBEBD)*-1/ISTD_SDBEBD as SLACK4_SDBEBD label="IStandardized Senior Debt to EBITDA Slack",
			 (c.SDBEBD-b.TARGET_SDBEBD)*-1/b.TARGET_SDBEBD as SLACK5_SDBEBD label="Percent Senior Debt to EBITDA Slack",

			 b.INITIAL_DBAT,
			 b.TARGET_DBAT,
			 c.DBAT as ACTUAL_DBAT,
			 c.STD_DBAT,
			 (c.DBAT-b.TARGET_DBAT)*-1/STD_DBAT as SLACK1_DBAT label="Standardized Leverage Ratio Slack",
			 (c.DBAT-b.TARGET_DBAT)*-1 as SLACK2_DBAT label="Leverage Ratio Slack",
			 -log(c.DBAT)+log(b.TARGET_DBAT) as SLACK3_DBAT label="Log Leverage Ratio Slack",
			 (c.DBAT-b.TARGET_DBAT)*-1/ISTD_DBAT as SLACK4_DBAT label="IStandardized Leverage Ratio Slack",
			 (c.DBAT-b.TARGET_DBAT)*-1/b.TARGET_DBAT as SLACK5_DBAT label="Percent Leverage Ratio Slack",

			 b.INITIAL_SDBAT,
			 b.TARGET_SDBAT,
			 c.SDBAT as ACTUAL_SDBAT,
			 c.STD_SDBAT,
			 (c.SDBAT-b.TARGET_SDBAT)*-1/STD_SDBAT as SLACK1_SDBAT label="Standardized Senior Leverage Ratio Slack",
			 (c.SDBAT-b.TARGET_SDBAT)*-1 as SLACK2_SDBAT label="Senior Leverage Ratio Slack",
			 -log(c.SDBAT)+log(b.TARGET_SDBAT) as SLACK3_SDBAT label="Log Senior Leverage Ratio Slack",
			 (c.SDBAT-b.TARGET_SDBAT)*-1/ISTD_SDBAT as SLACK4_SDBAT label="IStandardized Senior Leverage Ratio Slack",
			 (c.SDBAT-b.TARGET_SDBAT)*-1/b.TARGET_SDBAT as SLACK5_SDBAT label="Percent Senior Leverage Ratio Slack",

			 b.INITIAL_DBTNW,
			 b.TARGET_DBTNW,
			 c.DBTNW as ACTUAL_DBTNW,
			 c.STD_DBTNW,
			 (c.DBTNW-b.TARGET_DBTNW)*-1/STD_DBTNW as SLACK1_DBTNW label="Standardized Debt to Tang Net Worth Slack",
			 (c.DBTNW-b.TARGET_DBTNW)*-1 as SLACK2_DBTNW label="Debt to Tang Net Worth Slack",
			 -log(c.DBTNW)+log(b.TARGET_DBTNW) as SLACK3_DBTNW label="Log Debt to Tang Net Worth Slack",
			 (c.DBTNW-b.TARGET_DBTNW)*-1/ISTD_DBTNW as SLACK4_DBTNW label="IStandardized Debt to Tang Net Worth Slack",
			 (c.DBTNW-b.TARGET_DBTNW)*-1/b.TARGET_DBTNW as SLACK5_DBTNW label="Percent Debt to Tang Net Worth Slack",

			 b.INITIAL_DBNW,
			 b.TARGET_DBNW,
			 c.DBNW as ACTUAL_DBNW,
			 c.STD_DBNW,
			 (c.DBNW-b.TARGET_DBNW)*-1/STD_DBNW as SLACK1_DBNW label="Standardized Debt to Net Worth Slack",
			 (c.DBNW-b.TARGET_DBNW)*-1 as SLACK2_DBNW label="Debt to Net Worth Slack",
			 -log(c.DBNW)+log(b.TARGET_DBNW) as SLACK3_DBNW label="Log Debt to Net Worth Slack",
			 (c.DBNW-b.TARGET_DBNW)*-1/ISTD_DBNW as SLACK4_DBNW label="IStandardized Debt to Net Worth Slack",
			 (c.DBNW-b.TARGET_DBNW)*-1/b.TARGET_DBNW as SLACK5_DBNW label="Percent Debt to Net Worth Slack",

          b.INITIAL_CRTO,
			 b.TARGET_CRTO,
			 c.CRTO as ACTUAL_CRTO,
			 c.STD_CRTO,
			 (c.CRTO-b.TARGET_CRTO)/STD_CRTO as SLACK1_CRTO label="Standardized Current Ratio Slack",
			 (c.CRTO-b.TARGET_CRTO) as SLACK2_CRTO label="Current Ratio Slack",
			 log(c.CRTO)-log(b.TARGET_CRTO) as SLACK3_CRTO label="Log Current Ratio Slack",
          (c.CRTO-b.TARGET_CRTO)/ISTD_CRTO as SLACK4_CRTO label="IStandardized Current Ratio Slack",
          (c.CRTO-b.TARGET_CRTO)/b.TARGET_CRTO as SLACK5_CRTO label="Percent Current Ratio Slack",

			 b.INITIAL_QRTO,
			 b.TARGET_QRTO,
			 c.QRTO as ACTUAL_QRTO,
			 c.STD_QRTO,
			 (c.QRTO-b.TARGET_QRTO)/STD_QRTO as SLACK1_QRTO label="Standardized Quick Ratio Slack",
			 (c.QRTO-b.TARGET_QRTO) as SLACK2_QRTO label="Quick Ratio Slack",
			 log(c.QRTO)-log(b.TARGET_QRTO) as SLACK3_QRTO label="Log Quick Ratio Slack",
			 (c.QRTO-b.TARGET_QRTO)/ISTD_QRTO as SLACK4_QRTO label="IStandardized Quick Ratio Slack",
			 (c.QRTO-b.TARGET_QRTO)/b.TARGET_QRTO as SLACK5_QRTO label="Percent Quick Ratio Slack",

			 b.INITIAL_EBD/1000000 as INITIAL_EBD,
			 b.TARGET_EBD/1000000 as TARGET_EBD,
			 c.EBD as ACTUAL_EBD,
			 c.STD_EBD,
			 (c.EBD-b.TARGET_EBD/1000000)/STD_EBD as SLACK1_EBD label="Standardized EBITDA Slack",
			 (c.EBD-b.TARGET_EBD/1000000) as SLACK2_EBD label="EBITDA Slack",
			 log(c.EBD)-log(b.TARGET_EBD/1000000) as SLACK3_EBD label="Log EBITDA Slack",
			 (c.EBD-b.TARGET_EBD/1000000)/ISTD_EBD as SLACK4_EBD label="IStandardized EBITDA Slack",
			 (c.EBD-b.TARGET_EBD/1000000)/(b.TARGET_EBD/1000000) as SLACK5_EBD label="Percent EBITDA Slack",

			 b.INITIAL_NW/1000000 as INITIAL_NW,
			 b.TARGET_NW/1000000 as TARGET_NW,
			 c.NW as ACTUAL_NW,
			 c.STD_NW,
			 (c.NW-b.TARGET_NW/1000000)/STD_NW as SLACK1_NW label="Standardized Net Worth Slack",
			 (c.NW-b.TARGET_NW/1000000) as SLACK2_NW label="Net Worth Slack",
			 log(c.NW)-log(b.TARGET_NW/1000000) as SLACK3_NW label="Log Net Worth Slack",
			 (c.NW-b.TARGET_NW/1000000)/ISTD_NW as SLACK4_NW label="IStandardized Net Worth Slack",
			 (c.NW-b.TARGET_NW/1000000)/(b.TARGET_NW/1000000) as SLACK5_NW label="Percent Net Worth Slack",


			 b.INITIAL_TNW/1000000 as INITIAL_TNW,
			 b.TARGET_TNW/1000000 as TARGET_TNW,
			 c.TNW as ACTUAL_TNW,
			 c.STD_TNW,
			 (c.TNW-b.TARGET_TNW/1000000)/STD_TNW as SLACK1_TNW label="Standardized Tangible Net Worth Slack",
			 (c.TNW-b.TARGET_TNW/1000000) as SLACK2_TNW label="Tangible Net Worth Slack",
			 log(c.TNW)-log(b.TARGET_TNW/1000000) as SLACK3_TNW label="Log Tangible Net Worth Slack",
			 (c.TNW-b.TARGET_TNW/1000000)/ISTD_TNW as SLACK4_TNW label="IStandardized Tangible Net Worth Slack",
			 (c.TNW-b.TARGET_TNW/1000000)/(b.TARGET_TNW/1000000) as SLACK5_TNW label="Percent Tangible Net Worth Slack"

   from dfirms6(drop=initial_icvr--target_tnw) as a,
        dfirms6 as b,
		compq2 as c
	where a.packageid=b.packageid
    and a.gvkey = b.gvkey = c.gvkey
	 and a.fdateq = b.fdateq = c.fdateq;
	quit;




*compute Demiriglu and James SLACK variable for CRTO and DBEBD;
data DJtemp1;
   set dfirms7(keep=packageid etime TARGET_CRTO ACTUAL_CRTO);
   where ETIME=0 and TARGET_CRTO > . and ACTUAL_CRTO > .;
   if .25 <= ACTUAL_CRTO < .5 then DJTARGET_CRTO = .25;
   else if .50 <= ACTUAL_CRTO < .75 then DJTARGET_CRTO = .375;
   else if .75 <= ACTUAL_CRTO < 1.0 then DJTARGET_CRTO = .5;
   else if 1.0 <= ACTUAL_CRTO < 1.25 then DJTARGET_CRTO = .625;
   else if 1.25 <= ACTUAL_CRTO < 1.5 then DJTARGET_CRTO = .75;
   else if 1.50 <= ACTUAL_CRTO < 1.75then DJTARGET_CRTO = .875;
   else if 1.75 <= ACTUAL_CRTO < 2 then DJTARGET_CRTO = 1;
   else if 2.0 <= ACTUAL_CRTO < 2.25 then DJTARGET_CRTO = 1.125;
   else if 2.25 <= ACTUAL_CRTO < 2.5 then DJTARGET_CRTO = 1.25;
   else if 2.50 <= ACTUAL_CRTO < 2.75 then DJTARGET_CRTO = 1.375;
   else if 2.75 <= ACTUAL_CRTO < 3 then DJTARGET_CRTO = 1.5;
   else if 3.0 <= ACTUAL_CRTO < 3.25 then DJTARGET_CRTO = 1.625;
   else if 3.25 <= ACTUAL_CRTO < 3.5 then DJTARGET_CRTO = 1.75;
   else if 3.5 <= ACTUAL_CRTO < 3.75 then DJTARGET_CRTO = 1.875;
   else if 3.75 <= ACTUAL_CRTO < 4 then DJTARGET_CRTO = 2;
   else if 4.0 <= ACTUAL_CRTO < 4.25 then DJTARGET_CRTO = 2.125;
   else if 4.25 <= ACTUAL_CRTO < 4.5 then DJTARGET_CRTO = 2.25;
   else if 4.50 <= ACTUAL_CRTO  then DJTARGET_CRTO = 2.375;
   run;

data DJtemp1;
   set DJtemp1;
   if TARGET_CRTO < DJTARGET_CRTO then SLACK6_CRTO = "TIGHT";
      else SLACK6_CRTO = "LOOSE";
   run;

proc freq data=djtemp1;
   tables SLACK6_CRTO;
   run;

proc sql;
   create table dfirms7
   as select a.*,
             b.SLACK6_CRTO label="Demiriglu and James SLACK"
   from dfirms7 as a left join djtemp1 as b
   on a.packageid = b.packageid
    and a.etime = b.etime;
   quit;




*require firms to have data the quarter of contracting, and require that the SLACK be within 100% of the target on the negative end;
/*data temp;
   set dfirms7;
   where etime=0;
   GOOD_ICVR = -1 <= SLACK5_ICVR;
   GOOD_CICVR = -1 <= SLACK5_CICVR;
   GOOD_FCVR = -1 <= SLACK5_FCVR;
   GOOD_DSCVR = -1 <= SLACK5_DSCVR;
   GOOD_DBEBD = -1 <= SLACK5_DBEBD;
   GOOD_SDBEBD = -1 <= SLACK5_SDBEBD;
   GOOD_DBAT = -1 <= SLACK5_DBAT;
   GOOD_SDBAT = -1 <= SLACK5_SDBAT;
   GOOD_DBTNW = -1 <= SLACK5_DBTNW;
   GOOD_DBNW = -1 <= SLACK5_DBNW;
   GOOD_CRTO = -1 <= SLACK5_CRTO;
   GOOD_QRTO = -1 <= SLACK5_QRTO;
   GOOD_EBD = -1 <= SLACK5_EBD;
   GOOD_NW = -1 <= SLACK5_NW;
   GOOD_TNW = -1 <= SLACK5_TNW;
   keep packageid gvkey fdateq etime GOOD_:;
   run;
*/
data temp;
   set dfirms7;
   where etime=0;
   BAD_ICVR = . < SLACK5_ICVR < -1;
   BAD_CICVR = . < SLACK5_CICVR < -1;
   BAD_FCVR = . < SLACK5_FCVR < -1;
   BAD_DSCVR = . < SLACK5_DSCVR < -1;
   BAD_DBEBD = . < SLACK5_DBEBD < -1;
   BAD_SDBEBD = . < SLACK5_SDBEBD < -1;
   BAD_DBAT = . < SLACK5_DBAT < -1;
   BAD_SDBAT = . < SLACK5_SDBAT < -1;
   BAD_DBTNW = . < SLACK5_DBTNW < -1;
   BAD_DBNW = . < SLACK5_DBNW < -1;
   BAD_CRTO = . < SLACK5_CRTO < -1;
   BAD_QRTO = . < SLACK5_QRTO < -1;
   BAD_EBD = . < SLACK5_EBD < -1;
   BAD_NW = . < SLACK5_NW < -1;
   BAD_TNW = . < SLACK5_TNW < -1;
   keep packageid gvkey fdateq etime BAD_:;
   run;



proc sort data=temp nodupkey dupout=checkme;
   by packageid;
   run;


proc sql;
   create table dfirms7
   as select dfirms7.*,
             temp.*
   from dfirms7,temp
   where dfirms7.packageid = temp.packageid;
   quit;

              


data dfirms7;
   set dfirms7;
   *if sum(of BAD_:) = 0 then do;
   SLACK1 = min(of SLACK1_:);
   SLACK2 = min(of SLACK2_:);
   SLACK5 = min(of SLACK5_:);
   *end;
run;





data dfirms7;
   set dfirms7;
   by packageid fdateq;
   if first.packageid then VIOLATED=0;
   if ETIME <= 0 then VIOLATED=0;
   if ETIME >0 and . < SLACK2 < 0 and VIOLATED = 0 then VIOLATED=VIOLATED+1;
      else VIOLATED=VIOLATED;
   retain VIOLATED;
   run;
*get ETIMEV -- Event time leading up to first estimated violation;
*get quarter number;
proc sql;
   create table dfirms7
   as select a.*,
             intck('QTR',min(fdateq),fdateq) as QNUM
   from dfirms7 as a
   group by packageid
   order by packageid,fdateq;
   quit;
proc sql;
   create table temp8
   as select a.*
   from dfirms7 as a
   where VIOLATED=1
   group by packageid
   having datadate = min(datadate)
   order by packageid;
   quit;


*get ETIMEV;
proc sql;
   create table dfirms7(drop=QNUM)
   as select a.*,
             a.QNUM-b.QNUM as ETIMEV label="Event Time Leading to First Estimated Violation"
   from dfirms7 as a left join temp8 as b
    on a.packageid = b.packageid
   order by packageid,fdateq;
   quit;

data dfirms7;
   set dfirms7;
   DEAD = (ETIMEV >= 0) or (. < AMENDDATE <= DATADATE) or (. < NDEALDATE <= DATADATE) or (. < FACILITYENDDATE < DATADATE);
   run;
*get ETIMED -- Event time leading up estimated loan death;
*get quarter number;
proc sql;
   create table dfirms7
   as select a.*,
             intck('QTR',min(fdateq),fdateq) as QNUM
   from dfirms7 as a
   group by packageid
   order by packageid,fdateq;
   quit;
proc sql;
   create table temp8
   as select a.*
   from dfirms7 as a
   where DEAD=1
   group by packageid
   having datadate = min(datadate)
   order by packageid;
   quit;


*get ETIMED;
proc sql;
   create table dfirms7(drop=QNUM DEAD)
   as select a.*,
             a.QNUM-b.QNUM as ETIMED label="Event Time Leading to Estimated Loan Death"
   from dfirms7 as a left join temp8 as b
    on a.packageid = b.packageid
   order by packageid,fdateq;
   quit;





*figure out which covenant binds (ie which is tightest);
data dfirms7;
   set dfirms7;
   array covs(*) SLACK2_ICVR SLACK2_CICVR SLACK2_FCVR SLACK2_DSCVR SLACK2_DBEBD SLACK2_SDBEBD SLACK2_DBAT SLACK2_SDBAT SLACK2_DBTNW SLACK2_DBNW SLACK2_CRTO SLACK2_QRTO SLACK2_EBD SLACK2_NW SLACK2_TNW;
   array binds(*) BIND_ICVR BIND_CICVR BIND_FCVR BIND_DSCVR BIND_DBEBD BIND_SDBEBD BIND_DBAT BIND_SDBAT BIND_DBTNW BIND_DBNW BIND_CRTO BIND_QRTO BIND_EBD BIND_NW BIND_TNW;
   do i=1 to dim(covs);
      if covs(i) > . and covs(i)=SLACK2 then binds(i)=1;
	    else binds(i)=0;
	  if SLACK2 = . then binds(i)=.;
   end;
   drop i;
   run;



proc sql;
   create table temp
   as select PACKAGEID,
             datadate as VDATE label="Date of Estimated Violation" format=date9.
   from dfirms7
   where ETIMEV=0;
   quit;
proc sort data=temp noduplicates;
   by PACKAGEID;
   run;

proc sql;
   create table dfirms7
   as select dfirms7.*,
             temp.VDATE
   from dfirms7 left join temp
   on dfirms7.PACKAGEID = temp.PACKAGEID
   order by PACKAGEID,fdateq;
   quit;



*read in the violation data from Sufi;
data V(drop=datadate);
   set home.cstatviolations_nss_20090701;
   DAY=substr(datadate,3,2)*1;
   MONTH=substr(datadate,1,2)*1;
   YEAR = substr(datadate,5,4)*1;
   run;
*do a little data manipulation;
proc sql;
   create table V
   as select put(gvkey,z6.) as GVKEY,
			 mdy(month,day,year) as DATADATE format=date9.,
          1 as SUFIFIRM label="Indicates the Firm is in the Sufi Violation Sample",
             viol as V label="Indicates a Sufi Violation This Period"
   from V
   order by gvkey,datadate;
   quit;
proc sort data=V nodupkey;
   by gvkey datadate;
   run;


*merge into the dealscan data;
proc sql;
   create table temp
   as select a.*,
             b.SUFIFIRM,
             b.V label="Sufi Violation"
   from dfirms7 as a left join V as b
   on a.gvkey = b.gvkey
    and a.datadate = b.datadate
   order by packageid,fdateq;
	quit;


proc printto  log=junk; run;
proc expand data=temp method=none out=temp(where=(datadate ne .)) from=qtr align=end;
   by packageid;
	id fdateq;
	convert datadate=datadate;
	convert V  = V_lag1 / transformout = (lag 1) method=none;
	convert V  = V_lag2 / transformout = (lag 2) method=none;
	convert V  = V_lag3 / transformout = (lag 3) method=none;
	convert V  = V_lag4 / transformout = (lag 4) method=none;
	convert V  = V_lag5 / transformout = (lag 5) method=none;
	convert V  = V_lag6 / transformout = (lag 6) method=none;
	convert V  = V_lag7 / transformout = (lag 7) method=none;
	convert V  = V_lag8 / transformout = (lag 8) method=none;
	convert V  = V_lead1 / transformout = (lead 1) method=none;
	convert V  = V_lead2 / transformout = (lead 2) method=none;
	convert V  = V_lead3 / transformout = (lead 3) method=none;
	convert V  = V_lead4 / transformout = (lead 4) method=none;
	convert V  = V_lead5 / transformout = (lead 5) method=none;
	convert V  = V_lead6 / transformout = (lead 6) method=none;
	convert V  = V_lead7 / transformout = (lead 7) method=none;
	convert V  = V_lead8 / transformout = (lead 8) method=none;
	run;
proc printto;run;

data V2;
   set temp;
   if V=1 & V_lag1<1 & V_lag2<1 & V_lag3<1 & V_lag4<1 then NEWV=1;
     else NEWV=0;
   if V = . then NEWV = .;
   if NEWV=1 then NVdate=fdateq;
   format NVdate date9.;
   if sum(v,v_lag1,v_lag2,v_lag3,v_lag4,v_lead1,v_lead2,v_lead3,v_lead4)<1 then CONTROL=1;
      else CONTROL=0;
   label CONTROL = "Firm is Four Quarters Away from Sufi Violation"
         NVDATE = "Firm had a 'New' Sufi Violation This Quarter";
   drop V_lag1--V_lead8;
   run;
*get indicator for firms that violate at least once;
proc sql;
   create table V2
   as select V2.*,
             sum(V) > 0 as VIOLATOR label="Firm is a Sufi Violator"
   from V2
   group by packageid
   order by fdateq;
   quit;
proc sort data=V2;
   by fdateq;
   run;
proc download data=V2 out=work.V2;
   run;
proc download data=dfirms out=work.dfirms;
   run;
endrsubmit;

data DEALSCAN.DEALDATA_22DEC2014;
   set V2;
   if missing(sufifirm) then sufifirm=0;
   if V = . then do;
       CONTROL = .;
	   VIOLATOR = .;
	   end;
   run;

 
*get table with all loans, and slack at initiation (quarter ended closest to, but before deal start date);
proc sql;
   create table dealscan.ALLLOANS_22DEC2014(drop=etime)
   as select a.*,
             b.*
   from dfirms(keep=GVKEY--NCOV) as a left join DEALSCAN.DEALDATA_22DEC2014(where=(etime=0) keep=fdateq packageid etime initial_capx--VIOLATOR) as b
   on a.packageid = b.packageid;
   quit;




